
# StandaloneIpopt.jl

[![builds.sr.ht status](https://builds.sr.ht/~cgeoga/StandaloneIpopt.jl.svg)](https://builds.sr.ht/~cgeoga/StandaloneIpopt.jl?)

A simple package that uses `Ipopt.jl` to solve constrained nonlinear
optimization problems. See the examples and test files for usage demonstrations.

For the moment, it is very autodiff-oriented. Two functions are exported:

* `ipopt_optimize(objective, ini; [kwargs...])`, which uses some AD tool of your
  choice (between `ForwardDiff` and `ReverseDiff+ForwardDiff`) to get gradients
  and Hessians (unless you use Ipopt's BFGS approximation).

* `ipopt_nlsolve(fn, ini; [kwargs...])`, which solves the nonlinear system of
   equations `fn(v) = 0`. Again, AD is used for the Jacobian.

If you like the interface but want to provide your own gradient and/or
Lagrangian Hessian, let me know and I can put it in.

# Features

* Optional mixed-mode (forward-over-reverse) AD. It compiles the reverse-mode
  tapes, so even if you pass in an ugly and inefficient allocating objective
  you'll get nice gradients and Hessians. On the flip side, if compiling the tape
  doesn't work for you, I don't have much to offer for mixed-mode at present and
  you're stuck with just forward mode.

* Sparse constraint AD-jacobians, via `SparseDiffTools.jl`. Just pass in the
  `jac_sparsity` kwarg as a `SparseMatrixCSC{Int64, Bool}`.

* Pass in any Ipopt argument as a normal kwarg: `ipopt_optimize(obj, ini, [...],
  alpha_red_factor=0.15, [...])`.

# Anti-Features

* It does not depend on any very large or complex packages. No `Zygote` or
  `JuMP`. Those are both great packages. But there are a lot of nice tools to
  use those already. This one is really about simplicity while keeping the
  particular extra features that I (Chris Geoga) happen to want. 

# Disclaimer

I can't promise this will be a good fit for you, and I have wrapped it up
primarily for my own convenience. But I wouldn't be surprised if it were useful
to more people than just me, so hopefully it is convenient to use as packaged.
It's also on the MIT license and I welcome suggestions and/or enhancements.
Sourcehut uses email and not PRs, but you can also just email me a patch file. I
don't expect too too much traffic here.

# Contact

My email address(es) are listed on my personal website, which you can find with
any search engine.

