
# TODO (cg 2022/07/12 13:42): Write another test where the jacobian is actually
# sparse.

# Objective:
obj(x) = x[1]*x[4] * (x[1]+x[2]+x[3]) + x[3]

# Constraints:
if !(@isdefined ini)
  const con1 = [25.0, 1e22] => (x -> prod(x))
  const con2 = [40.0, 40.0] => (x -> sum(abs2, x))
  const (b_l, b_u)  = (1.0, 5.0)
  const ini = [1.0, 5.0, 5.0, 1.0]
  const expected_result = [1.000, 4.743, 3.821, 1.379]
end

for jsparse in (nothing, sparse(ones(Bool, 2, 4)))

  # Bufs for testing:
  g_buf_1 = zeros(4)
  g_buf_2 = zeros(4)
  con_jac_buf_1 = zeros(2*4)
  con_jac_buf_2 = zeros(2*4)
  lagrange_h_buf_1 = zeros(2*5)
  lagrange_h_buf_2 = zeros(2*5)

  # structure vectors for row and column of jacobian:
  jac_r = Int32[1,1,1,1,2,2,2,2]
  jac_c = Int32[1,2,3,4,1,2,3,4]

  # internal components:
  (my_obj, my_cons, my_objg, my_constr_jac, my_lagrange_h) = ipopt_optimize(obj, 
    ini, (con1, con2); jac_sparsity=jsparse, box_lower=b_l, 
    box_upper=b_u, return_internals=true)

  # Call my values:
  my_objg(ini, g_buf_1)
  my_constr_jac(ini, jac_r, jac_c, con_jac_buf_1)
  my_lagrange_h(ini, Int32[], Int32[], 1.5, [2.5, 3.5], lagrange_h_buf_1)

  # Now compare with the reference:
  eval_grad_f(ini, g_buf_2)
  eval_jac_g(ini, jac_r, jac_c, con_jac_buf_2)
  eval_h(ini, Int32[], Int32[], 1.5, [2.5, 3.5], lagrange_h_buf_2)

  # Test the individual functions:
  @test isapprox(g_buf_1, g_buf_2)
  @test isapprox(con_jac_buf_1, con_jac_buf_2)
  @test isapprox(lagrange_h_buf_1, lagrange_h_buf_2)

  # Test the optimization result:
  res = ipopt_optimize(obj, ini, (con1, con2), jac_sparsity=jsparse,
                       box_lower=b_l, box_upper=b_u, print_level=0)

  @test iszero(res.status)
  @test isapprox(res.minimizer, expected_result, atol=1e-3)

  # mixed-mode AD:
  if isnothing(jsparse)
    res_mixed = ipopt_optimize(obj, ini, (con1, con2), jac_sparsity=jsparse,
                               mixed_ad=true, box_lower=b_l, 
                               box_upper=b_u, print_level=0)
    @test iszero(res_mixed.status)
    @test isapprox(res_mixed.minimizer, expected_result, atol=1e-3)
  end

end

